@@ -155,7 +155,11 @@ g_vfs_decode_uri (const char *uri)
 	      authority   = [ userinfo "@" ] host [ ":" port ]
       */
 
-      userinfo_end = memchr (authority_start, '@', authority_end - authority_start);
+      /* Look for the last so that any multiple @ signs are put in the username part.
+       * This is not quite correct, as @ should be escaped here, but this happens
+       * in practice, so lets handle it the "nicer" way at least. */
+      userinfo_end = g_strrstr_len (authority_start,
+				    authority_end - authority_start, "@");
       if (userinfo_end)
 	{
 	  userinfo_start = authority_start;
@@ -225,7 +229,7 @@ g_vfs_decode_uri (const char *uri)
 	  decoded->port = -1;
 	}
 
-      decoded->host = g_strndup (host_start, host_end - host_start);
+      decoded->host = g_uri_unescape_segment (host_start, host_end, NULL);
 
       hier_part_start = authority_end;
     }
@@ -261,7 +265,10 @@ g_vfs_encode_uri (GDecodedUri *decoded, gboolean allow_utf8)
 	  g_string_append_c (uri, '@');
 	}
       
-      g_string_append (uri, decoded->host);
+      g_string_append_uri_escaped (uri, decoded->host,
+				   /* Allowed unescaped in hostname / ip address */
+				   G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS ":[]" ,
+				   allow_utf8);
       
       if (decoded->port != -1)
 	{
