@@ -40,6 +40,7 @@
 #include <gvfsdaemonutils.h>
 #include <gvfsjobcloseread.h>
 #include <gvfsjobclosewrite.h>
+#include <gvfsfileinfo.h>
 
 static void g_vfs_channel_job_source_iface_init (GVfsJobSourceIface *iface);
 
@@ -84,7 +85,6 @@ struct _GVfsChannelPrivate
   GInputStream *command_stream;
   GOutputStream *reply_stream;
   int remote_fd;
-  gboolean cancelled;
   
   GVfsBackendHandle backend_handle;
   GVfsJob *current_job;
@@ -341,8 +341,7 @@ got_request (GVfsChannel *channel,
   command = g_ntohl (request->command);
   arg1 = g_ntohl (request->arg1);
 
-  if (command == G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_CANCEL ||
-      channel->priv->cancelled)
+  if (command == G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_CANCEL)
     {
       if (arg1 == channel->priv->current_job_seq_nr &&
 	  channel->priv->current_job != NULL)
@@ -366,8 +365,6 @@ got_request (GVfsChannel *channel,
 	    }
 	}
 
-      channel->priv->cancelled = TRUE;
-      
       /* Cancel ops get no return */
       g_free (data);
       return;
@@ -608,8 +605,7 @@ send_reply_cb (GObject *source_object,
       g_vfs_job_source_new_job (G_VFS_JOB_SOURCE (channel), channel->priv->current_job);
     }
   /* Start queued request or readahead */
-  else if (!channel->priv->cancelled &&
-	   !start_queued_request (channel) &&
+  else if (!start_queued_request (channel) &&
 	   class->readahead)
     {
       /* No queued requests, maybe we want to do a readahead call */
@@ -669,6 +665,26 @@ g_vfs_channel_send_error (GVfsChannel *channel,
   g_vfs_channel_send_reply (channel, NULL, data, data_len);
 }
 
+/* Might be called on an i/o thread
+ */
+void
+g_vfs_channel_send_info (GVfsChannel *channel,
+			 GFileInfo *info)
+{
+  GVfsDaemonSocketProtocolReply reply;
+  char *data;
+  gsize data_len;
+  
+  data = gvfs_file_info_marshal (info, &data_len);
+
+  reply.type = g_htonl (G_VFS_DAEMON_SOCKET_PROTOCOL_REPLY_INFO);
+  reply.seq_nr = g_htonl (g_vfs_channel_get_current_seq_nr (channel));
+  reply.arg1 = 0;
+  reply.arg2 = g_htonl (data_len);
+
+  g_vfs_channel_send_reply (channel, &reply, data, data_len);
+}
+
 int
 g_vfs_channel_steal_remote_fd (GVfsChannel *channel)
 {
